@@ -74,6 +74,7 @@ struct _GVfsBackendPrivate
   char *stable_name;
   char **x_content_types;
   GIcon *icon;
+  GIcon *symbolic_icon;
   char *prefered_filename_encoding;
   gboolean user_visible;
   char *default_location;
@@ -148,6 +149,7 @@ g_vfs_backend_finalize (GObject *object)
   g_free (backend->priv->stable_name);
   g_strfreev (backend->priv->x_content_types);
   g_clear_object (&backend->priv->icon);
+  g_clear_object (&backend->priv->symbolic_icon);
   g_free (backend->priv->prefered_filename_encoding);
   g_free (backend->priv->default_location);
   if (backend->priv->mount_spec)
@@ -197,6 +199,7 @@ g_vfs_backend_init (GVfsBackend *backend)
 {
   backend->priv = G_TYPE_INSTANCE_GET_PRIVATE (backend, G_VFS_TYPE_BACKEND, GVfsBackendPrivate);
   backend->priv->icon = NULL;
+  backend->priv->symbolic_icon = NULL;
   backend->priv->prefered_filename_encoding = g_strdup ("");
   backend->priv->display_name = g_strdup ("");
   backend->priv->stable_name = g_strdup ("");
@@ -399,6 +402,22 @@ g_vfs_backend_set_icon (GVfsBackend *backend,
   backend->priv->icon = g_object_ref (icon);
 }
 
+void
+g_vfs_backend_set_symbolic_icon_name (GVfsBackend *backend,
+                                      const char *icon_name)
+{
+  g_clear_object (&backend->priv->symbolic_icon);
+  backend->priv->symbolic_icon = g_themed_icon_new_with_default_fallbacks (icon_name);
+}
+
+void
+g_vfs_backend_set_symbolic_icon (GVfsBackend *backend,
+                                 GIcon       *icon)
+{
+  g_clear_object (&backend->priv->symbolic_icon);
+  backend->priv->symbolic_icon = g_object_ref (icon);
+}
+
 void
 g_vfs_backend_set_prefered_filename_encoding (GVfsBackend  *backend,
 					      const char *prefered_filename_encoding)
@@ -475,6 +494,12 @@ g_vfs_backend_get_icon (GVfsBackend *backend)
   return backend->priv->icon;
 }
 
+GIcon *
+g_vfs_backend_get_symbolic_icon (GVfsBackend *backend)
+{
+  return backend->priv->symbolic_icon;
+}
+
 const char *
 g_vfs_backend_get_default_location (GVfsBackend  *backend)
 {
@@ -632,9 +657,10 @@ register_mount_got_proxy_cb (GObject *source_object,
   GError *error = NULL;
   GSimpleAsyncResult *result;
   GVfsBackend *backend;
-  const char *stable_name;
+  char *stable_name;
   char *x_content_types_string;
   char *icon_str;
+  char *symbolic_icon_str;
 
   proxy = gvfs_dbus_mount_tracker_proxy_new_for_bus_finish (res, &error);
   if (proxy == NULL)
@@ -662,11 +688,12 @@ register_mount_got_proxy_cb (GObject *source_object,
   else
     icon_str = g_strdup ("");
 
-  if (backend->priv->stable_name != NULL &&
-      *backend->priv->stable_name != 0)
-   stable_name = backend->priv->stable_name;
+  if (backend->priv->symbolic_icon != NULL)
+    symbolic_icon_str = g_icon_to_string (backend->priv->symbolic_icon);
   else
-   stable_name = backend->priv->display_name;
+    symbolic_icon_str = g_strdup ("");
+
+  stable_name = g_mount_spec_to_string (backend->priv->mount_spec);
 
   gvfs_dbus_mount_tracker_call_register_mount (proxy,
                                                backend->priv->object_path,
@@ -674,6 +701,7 @@ register_mount_got_proxy_cb (GObject *source_object,
                                                stable_name,
                                                x_content_types_string,
                                                icon_str,
+                                               symbolic_icon_str,
                                                backend->priv->prefered_filename_encoding,
                                                backend->priv->user_visible,
                                                g_mount_spec_to_dbus (backend->priv->mount_spec),
@@ -681,8 +709,10 @@ register_mount_got_proxy_cb (GObject *source_object,
                                                NULL,
                                                data->callback, data->callback_data);
 
+  g_free (stable_name);
   g_free (x_content_types_string);
   g_free (icon_str);
+  g_free (symbolic_icon_str);
   g_object_unref (proxy);
   async_proxy_create_free (data);
 }
@@ -831,7 +861,6 @@ on_update_processes_timeout (gpointer user_data)
                                            data->message,
                                            processes,
                                            data->choices,
-                                           g_strv_length ((gchar **) data->choices),
                                            NULL,
                                            NULL);
     }
@@ -952,7 +981,6 @@ g_vfs_backend_unmount_with_operation (GVfsBackend        *backend,
                                        data->message,
                                        processes,
                                        data->choices,
-                                       g_strv_length ((gchar **) data->choices),
                                        (GAsyncReadyCallback) on_show_processes_reply,
                                        data);
 
