@@ -467,6 +467,10 @@ void        g_vfs_backend_set_icon_name                  (GVfsBackend        *ba
 							  const char         *icon_name);
 void        g_vfs_backend_set_icon                       (GVfsBackend        *backend,
 							  GIcon              *icon);
+void        g_vfs_backend_set_symbolic_icon_name         (GVfsBackend        *backend,
+							  const char         *icon_name);
+void        g_vfs_backend_set_symbolic_icon              (GVfsBackend        *backend,
+							  GIcon              *icon);
 void        g_vfs_backend_set_prefered_filename_encoding (GVfsBackend        *backend,
 							  const char         *prefered_filename_encoding);
 void        g_vfs_backend_set_user_visible               (GVfsBackend        *backend,
@@ -486,6 +490,7 @@ const char *g_vfs_backend_get_display_name               (GVfsBackend        *ba
 const char *g_vfs_backend_get_stable_name                (GVfsBackend        *backend);
 char      **g_vfs_backend_get_x_content_types            (GVfsBackend        *backend);
 GIcon      *g_vfs_backend_get_icon                       (GVfsBackend        *backend);
+GIcon      *g_vfs_backend_get_symbolic_icon              (GVfsBackend        *backend);
 const char *g_vfs_backend_get_default_location           (GVfsBackend        *backend);
 GMountSpec *g_vfs_backend_get_mount_spec                 (GVfsBackend        *backend);
 GVfsDaemon *g_vfs_backend_get_daemon                     (GVfsBackend        *backend);
