@@ -485,10 +485,10 @@ restart:
         {
           if (ftp->has_initial_user)
             /* Translators: the first %s is the username, the second the host name */
-            prompt = g_strdup_printf (_("Enter password for FTP as %s on %s"), ftp->user, ftp->host_display_name);
+            prompt = g_strdup_printf (_("Enter password for %s on %s"), ftp->user, ftp->host_display_name);
           else
             /* translators: %s here is the hostname */
-            prompt = g_strdup_printf (_("Enter password for FTP on %s"), ftp->host_display_name);
+            prompt = g_strdup_printf (_("Enter password for %s"), ftp->host_display_name);
         }
          
       flags = G_ASK_PASSWORD_NEED_PASSWORD;
@@ -627,11 +627,11 @@ try_login:
     g_mount_spec_set (mount_spec, "user", ftp->user);
      
   if (g_str_equal (ftp->user, "anonymous"))
-    display_name = g_strdup_printf (_("FTP on %s"), ftp->host_display_name);
+    display_name = g_strdup (ftp->host_display_name);
   else
     {
       /* Translators: the first %s is the username, the second the host name */
-      display_name = g_strdup_printf (_("FTP as %s on %s"), ftp->user, ftp->host_display_name);
+      display_name = g_strdup_printf (_("%s on %s"), ftp->user, ftp->host_display_name);
     }
   g_vfs_backend_set_mount_spec (backend, mount_spec);
   g_mount_spec_unref (mount_spec);
@@ -639,6 +639,7 @@ try_login:
   g_vfs_backend_set_display_name (backend, display_name);
   g_free (display_name);
   g_vfs_backend_set_icon_name (backend, "folder-remote");
+  g_vfs_backend_set_symbolic_icon_name (backend, "folder-remote-symbolic");
 
   ftp->connections = 1;
   ftp->max_connections = G_MAXUINT;
