@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Author: Alexander Larsson <alexl@redhat.com>
  */
@@ -28,11 +28,10 @@
 #include <sys/un.h>
 
 #include <glib.h>
-#include <dbus/dbus.h>
 #include <glib/gi18n.h>
+#include <gio/gunixfdlist.h>
 #include "gvfswritechannel.h"
 #include "gvfsjobopenforwrite.h"
-#include "gvfsdbusutils.h"
 #include "gvfsdaemonutils.h"
 
 G_DEFINE_TYPE (GVfsJobOpenForWrite, g_vfs_job_open_for_write, G_VFS_TYPE_JOB_DBUS)
@@ -40,9 +39,9 @@ G_DEFINE_TYPE (GVfsJobOpenForWrite, g_vfs_job_open_for_write, G_VFS_TYPE_JOB_DBU
 static void         run          (GVfsJob        *job);
 static gboolean     try          (GVfsJob        *job);
 static void         finished     (GVfsJob        *job);
-static DBusMessage *create_reply (GVfsJob        *job,
-				  DBusConnection *connection,
-				  DBusMessage    *message);
+static void         create_reply (GVfsJob               *job,
+                                  GVfsDBusMount         *object,
+                                  GDBusMethodInvocation *invocation);
 
 static void
 g_vfs_job_open_for_write_finalize (GObject *object)
@@ -82,60 +81,41 @@ g_vfs_job_open_for_write_init (GVfsJobOpenForWrite *job)
 {
 }
 
-GVfsJob *
-g_vfs_job_open_for_write_new (DBusConnection *connection,
-			      DBusMessage *message,
-			      GVfsBackend *backend)
+gboolean
+g_vfs_job_open_for_write_new_handle (GVfsDBusMount *object,
+                                     GDBusMethodInvocation *invocation,
+                                     GUnixFDList *fd_list,
+                                     const gchar *arg_path_data,
+                                     guint16 arg_mode,
+                                     const gchar *arg_etag,
+                                     gboolean arg_make_backup,
+                                     guint arg_flags,
+                                     guint arg_pid,
+                                     GVfsBackend *backend)
 {
   GVfsJobOpenForWrite *job;
-  DBusMessageIter iter;
-  DBusMessage *reply;
-  DBusError derror;
-  char *path;
-  guint16 mode;
-  dbus_bool_t make_backup;
-  const char *etag;
-  guint32 flags;
-  guint32 pid;
-
-  path = NULL;
-  dbus_error_init (&derror);
-  dbus_message_iter_init (message, &iter);
-  if (!_g_dbus_message_iter_get_args (&iter, &derror, 
-				      G_DBUS_TYPE_CSTRING, &path,
-				      DBUS_TYPE_UINT16, &mode,
-				      DBUS_TYPE_STRING, &etag,
-				      DBUS_TYPE_BOOLEAN, &make_backup,
-				      DBUS_TYPE_UINT32, &flags,
-                                      DBUS_TYPE_UINT32, &pid,
-				      0))
-    {
-      reply = dbus_message_new_error (message,
-				      derror.name,
-                                      derror.message);
-      dbus_error_free (&derror);
-
-      dbus_connection_send (connection, reply, NULL);
-      dbus_message_unref (reply);
-      g_free (path);
-      return NULL;
-    }
+  
+  if (g_vfs_backend_invocation_first_handler (object, invocation, backend))
+    return TRUE;
   
   job = g_object_new (G_VFS_TYPE_JOB_OPEN_FOR_WRITE,
-		      "message", message,
-		      "connection", connection,
-		      NULL);
-
-  job->filename = path;
-  job->mode = mode;
-  if (*etag != 0)
-    job->etag = g_strdup (etag);
-  job->make_backup = make_backup;
-  job->flags = flags;
-  job->backend = backend;
-  job->pid = pid;
+                      "object", object,
+                      "invocation", invocation,
+                      NULL);
   
-  return G_VFS_JOB (job);
+  job->filename = g_strdup (arg_path_data);
+  job->mode = arg_mode;
+  if (*arg_etag != 0)
+    job->etag = g_strdup (arg_etag);
+  job->make_backup = arg_make_backup;
+  job->flags = arg_flags;
+  job->backend = backend;
+  job->pid = arg_pid;
+
+  g_vfs_job_source_new_job (G_VFS_JOB_SOURCE (backend), G_VFS_JOB (job));
+  g_object_unref (job);
+
+  return TRUE;
 }
 
 static void
@@ -259,56 +239,48 @@ g_vfs_job_open_for_write_set_initial_offset (GVfsJobOpenForWrite *job,
   job->initial_offset = initial_offset;
 }
 
-/* Might be called on an i/o thwrite */
-static DBusMessage *
+/* Might be called on an i/o thread */
+static void
 create_reply (GVfsJob *job,
-	      DBusConnection *connection,
-	      DBusMessage *message)
+              GVfsDBusMount *object,
+              GDBusMethodInvocation *invocation)
 {
   GVfsJobOpenForWrite *open_job = G_VFS_JOB_OPEN_FOR_WRITE (job);
   GVfsWriteChannel *channel;
-  DBusMessage *reply;
   GError *error;
   int remote_fd;
   int fd_id;
-  dbus_bool_t can_seek;
-  guint64 initial_offset;
-  
+  GUnixFDList *fd_list;
+
   g_assert (open_job->backend_handle != NULL);
 
-  error = NULL;
   channel = g_vfs_write_channel_new (open_job->backend,
                                      open_job->pid);
 
   remote_fd = g_vfs_channel_steal_remote_fd (G_VFS_CHANNEL (channel));
-  if (!dbus_connection_send_fd (connection, 
-				remote_fd,
-				&fd_id, &error))
-    {
-      close (remote_fd);
-      reply = _dbus_message_new_from_gerror (message, error);
-      g_error_free (error);
-      g_object_unref (channel);
-      return reply;
-    }
-  close (remote_fd);
 
-  reply = dbus_message_new_method_return (message);
-  can_seek = open_job->can_seek;
-  initial_offset = open_job->initial_offset;
-  dbus_message_append_args (reply,
-			    DBUS_TYPE_UINT32, &fd_id,
-			    DBUS_TYPE_BOOLEAN, &can_seek,
-			    DBUS_TYPE_UINT64, &initial_offset,
-			    DBUS_TYPE_INVALID);
+  fd_list = g_unix_fd_list_new ();
+  error = NULL;
+  fd_id = g_unix_fd_list_append (fd_list, remote_fd, &error);
+  if (fd_id == -1)
+    {
+      g_warning ("create_reply: %s (%s, %d)\n", error->message, g_quark_to_string (error->domain), error->code);
+      g_error_free (error);
+    }
+
+  gvfs_dbus_mount_complete_open_for_write (object, invocation,
+                                           fd_list, g_variant_new_handle (fd_id),
+                                           open_job->can_seek,
+                                           open_job->initial_offset);
+  
+  close (remote_fd);
+  g_object_unref (fd_list);
 
   g_vfs_channel_set_backend_handle (G_VFS_CHANNEL (channel), open_job->backend_handle);
   open_job->backend_handle = NULL;
   open_job->write_channel = channel;
 
   g_signal_emit_by_name (job, "new-source", open_job->write_channel);
-  
-  return reply;
 }
 
 static void
